/*
 * scaleup.c - corrected Client/server model
 *
 * Grow the number of PC/Users by 2x and look for bottlenecks.
 *
 * Updated by njgunther@perfdynamics.com Thu Nov 15 09:24:33  2001
 * Revised by njgunther@perfdynamics.com Sun Nov 25 07:21:34  2001
 *
 *  $Id: scaleup.c,v 1.1 2004/10/05 10:15:36 zyx Exp $
 */

//-------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "PDQ_Lib.h"


//----- Model Parameters --------------------------------------------------
// Name of this model ...

char           *scenario = "C/S Scaleup";


// Useful multipliers ...

#define 	K	(float) 1024
#define 	MIPS 		1E6
#define 	Mbps 		1E6

// Model parameters ...

#define		USERS		250
#define		FS_DISKS	 1    
#define		MF_DISKS	 4
#define		PC_MIPS		(27 * MIPS)
#define		FS_MIPS		(41 * MIPS)
#define		GW_MIPS		(10 * MIPS)
#define		MF_MIPS		(21 * MIPS)
#define		TR_Mbps	 	(4 * Mbps)
#define		TR_fact	 	4

#define MAXPROC	20
#define MAXDEV	50

// Computing Device IDs

#define	PC	0
#define	FS	1
#define	GW	2
#define	MF	3
#define	TR	4
#define	FDA	10
#define	MDA	20

// Transaction IDs

#define	CD	0                   /* Category Display */
#define	RQ	1                   /* Remote Query */
#define	SU	2                   /* Status Update */

// Process IDs  from 1993 paper

#define	CD_Req	1
#define	CD_Rpy	15
#define	RQ_Req	2
#define	RQ_Rpy	16
#define	SU_Req	3
#define	SU_Rpy	17
#define	Req_CD	4
#define	Req_RQ	5
#define	Req_SU	6
#define	CD_Msg	12
#define	RQ_Msg	13
#define	SU_Msg	14
#define	GT_Snd	7
#define	GT_Rcv	11
#define	MF_CD	8
#define	MF_RQ	9
#define	MF_SU	10
#define	LAN_Tx	18

//-------------------------------------------------------------------------
// This should go into PDQ_Lib.h ...

typedef struct {
	int             id;
	char            label[MAXCHARS];
}               devarray_type;

//-------------------------------------------------------------------------

main()
{
	extern JOB_TYPE   *job;
	extern NODE_TYPE  *node;
	extern             DEBUG;
	extern char        s1[];

	int                noNodes;
	int                noStreams;

	char               txCD[MAXCHARS],
				          txRQ[MAXCHARS],
				          txSU[MAXCHARS];
	double             demand[MAXPROC][MAXDEV],
				          util[MAXDEV],
				          X;
	double             ulan,
				          ufs,
				          udsk[MAXDEV],
				          uws,
				          ugw,
				          umf,
				          udasd[MAXDEV];
	int                work,
				          dev,
				          i,
				          j;

	// This should go into PDQ_Build.c ...

	devarray_type  *FDarray;
	devarray_type  *MDarray;

	if ((FDarray = (devarray_type *) calloc(sizeof(devarray_type), 10)) == NULL)
		errmsg("", "FDarray allocation failed!\n");
	if ((MDarray = (devarray_type *) calloc(sizeof(devarray_type), 10)) == NULL)
		errmsg("", "MDarray allocation failed!\n");

	for (i = 0; i < FS_DISKS; i++) {
		FDarray[i].id = FDA + i;
		resets(s1);
		sprintf(s1, "FD%d", i);
		strcpy(FDarray[i].label, s1);
	}

	for (i = 0; i < MF_DISKS; i++) {
		MDarray[i].id = MDA + i;
		resets(s1);
		sprintf(s1, "MD%d", i);
		strcpy(MDarray[i].label, s1);
	}

	/*
	 * CPU service times are calculated from MIPS Instruction counts in
	 * tables presented in original 1993 CMG paper.
	 */

	demand[CD_Req][PC] = 200 * K / PC_MIPS;
	demand[CD_Rpy][PC] = 100 * K / PC_MIPS;
	demand[RQ_Req][PC] = 150 * K / PC_MIPS;
	demand[RQ_Rpy][PC] = 200 * K / PC_MIPS;
	demand[SU_Req][PC] = 300 * K / PC_MIPS;
	demand[SU_Rpy][PC] = 300 * K / PC_MIPS;

	demand[Req_CD][FS] = 50 * K / FS_MIPS;
	demand[Req_RQ][FS] = 70 * K / FS_MIPS;
	demand[Req_SU][FS] = 10 * K / FS_MIPS;
	demand[CD_Msg][FS] = 35 * K / FS_MIPS;
	demand[RQ_Msg][FS] = 35 * K / FS_MIPS;
	demand[SU_Msg][FS] = 35 * K / FS_MIPS;

	demand[GT_Snd][GW] = 50 * K / GW_MIPS;
	demand[GT_Rcv][GW] = 50 * K / GW_MIPS;

	demand[MF_CD][MF] = 50 * K / MF_MIPS;
	demand[MF_RQ][MF] = 150 * K / MF_MIPS;
	demand[MF_SU][MF] = 20 * K / MF_MIPS;

	// packets generated at each of the following sources ...

	demand[LAN_Tx][PC] = 2 * K * TR_fact / TR_Mbps;
	demand[LAN_Tx][FS] = 2 * K * TR_fact / TR_Mbps;
	demand[LAN_Tx][GW] = 2 * K * TR_fact / TR_Mbps;

	/* File server Disk I/Os = #accesses x caching / (max IOs/Sec) */

	for (i = 0; i < FS_DISKS; i++) {
		demand[Req_CD][FDarray[i].id] = (1.0 * 0.5 / 128.9) / FS_DISKS;
		demand[Req_RQ][FDarray[i].id] = (1.5 * 0.5 / 128.9) / FS_DISKS;
		demand[Req_SU][FDarray[i].id] = (0.2 * 0.5 / 128.9) / FS_DISKS;
		demand[CD_Msg][FDarray[i].id] = (1.0 * 0.5 / 128.9) / FS_DISKS;
		demand[RQ_Msg][FDarray[i].id] = (1.5 * 0.5 / 128.9) / FS_DISKS;
		demand[SU_Msg][FDarray[i].id] = (0.5 * 0.5 / 128.9) / FS_DISKS;
	}

	/* Mainframe DASD I/Os = (#accesses / (max IOs/Sec)) / #disks */

	for (i = 0; i < MF_DISKS; i++) {
		demand[MF_CD][MDarray[i].id] = (2.0 / 60.24) / MF_DISKS;
		demand[MF_RQ][MDarray[i].id] = (4.0 / 60.24) / MF_DISKS;
		demand[MF_SU][MDarray[i].id] = (1.0 / 60.24) / MF_DISKS;
	}


	// Start building the PDQ model  ...

	PDQ_Init(scenario);

	// Define physical resources as queues ...

	noNodes = PDQ_CreateNode("PC", CEN, FCFS);
	noNodes = PDQ_CreateNode("FS", CEN, FCFS);

	for (i = 0; i < FS_DISKS; i++) {
		noNodes = PDQ_CreateNode(FDarray[i].label, CEN, FCFS);
	}

	noNodes = PDQ_CreateNode("GW", CEN, FCFS);
	noNodes = PDQ_CreateNode("MF", CEN, FCFS);

	for (i = 0; i < MF_DISKS; i++) {
		noNodes = PDQ_CreateNode(MDarray[i].label, CEN, FCFS);
	}

	noNodes = PDQ_CreateNode("TR", CEN, FCFS);

	/*
	 * NOTE: Althought the Token Ring LAN is a passive computational device,
	 * it is treated as a separate node so as to agree to the results
	 * presented in the original CMG'93 paper.
	 * 
	 */

	// Assign transaction names ...

	strcpy(txCD, "CatDsply");
	strcpy(txRQ, "RemQuote");
	strcpy(txSU, "StatusUp");

	// Define an OPEN circuit aggregate workload ...

	noStreams = PDQ_CreateOpen(txCD, USERS * 4.0 / 60.0);
	noStreams = PDQ_CreateOpen(txRQ, USERS * 8.0 / 60.0);
	noStreams = PDQ_CreateOpen(txSU, USERS * 1.0 / 60.0);

	// Define the service demands on each physical resource ...
	// CD request + reply chain  from workflow diagram

	PDQ_SetDemand("PC", txCD, demand[CD_Req][PC] + (5 * demand[CD_Rpy][PC]));
	PDQ_SetDemand("FS", txCD, demand[Req_CD][FS] + (5 * demand[CD_Msg][FS]));

	for (i = 0; i < FS_DISKS; i++) {
		  PDQ_SetDemand(FDarray[i].label, txCD,
				          demand[Req_CD][FDarray[i].id] +
				          (5 * demand[CD_Msg][FDarray[i].id])
				);
	}

	PDQ_SetDemand("GW", txCD, demand[GT_Snd][GW] + (5 * demand[GT_Rcv][GW]));
	PDQ_SetDemand("MF", txCD, demand[MF_CD][MF]);

	for (i = 0; i < MF_DISKS; i++) {
		PDQ_SetDemand(MDarray[i].label, txCD,
		demand[MF_CD][MDarray[i].id]
		);
	}

	/*
	 *  NOTE: Synchronous process execution causes data for the CD
	 * transaction to cross the LAN 12 times as depicted in the following
	 * parameterization of PDQ_SetDemand.
	 */

	PDQ_SetDemand("TR", txCD,
				      (1 * demand[LAN_Tx][PC]) +
				      (1 * demand[LAN_Tx][FS]) +
				      (5 * demand[LAN_Tx][GW]) +
				      (5 * demand[LAN_Tx][FS])
		  );

	// RQ request + reply chain ...

	PDQ_SetDemand("PC", txRQ, demand[RQ_Req][PC] + (3 * demand[RQ_Rpy][PC]));
	PDQ_SetDemand("FS", txRQ, demand[Req_RQ][FS] + (3 * demand[RQ_Msg][FS]));

	for (i = 0; i < FS_DISKS; i++) {
		PDQ_SetDemand(FDarray[i].label, txRQ,
			demand[Req_RQ][FDarray[i].id] +
			(3 * demand[RQ_Msg][FDarray[i].id])
		);
	}

	PDQ_SetDemand("GW", txRQ, demand[GT_Snd][GW] + (3 * demand[GT_Rcv][GW]));
	PDQ_SetDemand("MF", txRQ, demand[MF_RQ][MF]);

	for (i = 0; i < MF_DISKS; i++) {
		PDQ_SetDemand(MDarray[i].label, txRQ,
			demand[MF_RQ][MDarray[i].id]
		);
	}

	PDQ_SetDemand("TR", txRQ,
				      (1 * demand[LAN_Tx][PC]) +
				      (1 * demand[LAN_Tx][FS]) +
				      (3 * demand[LAN_Tx][GW]) +
				      (3 * demand[LAN_Tx][FS])
		  );

	// SU request + reply chain  ...

	PDQ_SetDemand("PC", txSU, demand[SU_Req][PC] + demand[SU_Rpy][PC]);
	PDQ_SetDemand("TR", txSU, demand[LAN_Tx][PC]);
	PDQ_SetDemand("FS", txSU, demand[Req_SU][FS] + demand[SU_Msg][FS]);

	for (i = 0; i < FS_DISKS; i++) {
		PDQ_SetDemand(FDarray[i].label, txSU,
			demand[Req_SU][FDarray[i].id] +
			demand[SU_Msg][FDarray[i].id]
		);
	}

	PDQ_SetDemand("TR", txSU, demand[LAN_Tx][FS]);
	PDQ_SetDemand("GW", txSU, demand[GT_Snd][GW] + demand[GT_Rcv][GW]);
	PDQ_SetDemand("MF", txSU, demand[MF_SU][MF]);

	for (i = 0; i < MF_DISKS; i++) {
		PDQ_SetDemand(MDarray[i].label, txSU,
			demand[MF_SU][MDarray[i].id]);
	}

	PDQ_SetDemand("TR", txSU,
				      (1 * demand[LAN_Tx][PC]) +
				      (1 * demand[LAN_Tx][FS]) +
				      (1 * demand[LAN_Tx][GW]) +
				      (1 * demand[LAN_Tx][FS])
		  );


	DEBUG = FALSE;

	PDQ_Solve(CANON);

	PDQ_Report();

	// Break out Tx response times and device utilizations ...

	printf("*** PDQ Breakout \"%s\" (%d clients) ***\n\n",
		scenario, USERS);

	for (dev = 0; dev < noNodes; dev++) {
		util[dev] = 0.0;        /* reset array */
		for (work = 0; work < noStreams; work++) {
			util[dev] += 100 * PDQ_GetUtilization(
				node[dev].devname,
				job[work].trans->name,
				TRANS
			);
		}
	}

	// Order of print out follows that in 1993 CMG paper.

	printf("Transaction  \tLatency(Secs)\n");
	printf("-----------  \t-----------\n");

	for (work = 0; work < noStreams; work++) {
		printf("%s\t%7.4f\n",
			job[work].trans->name, job[work].trans->sys->response);
	}

	printf("\n\n");

	for (dev = 0; dev < noNodes; dev++) {
		if (strcmp(node[dev].devname, "PC") == 0)
			uws += util[dev];
		if (strcmp(node[dev].devname, "GW") == 0)
			ugw += util[dev];
		if (strcmp(node[dev].devname, "FS") == 0)
			ufs += util[dev];

		for (i = 0; i < FS_DISKS; i++) {
			if (strcmp(node[dev].devname, FDarray[i].label) == 0)
				udsk[i] += util[dev];
		}

		if (strcmp(node[dev].devname, "MF") == 0)
			umf += util[dev];

		for (i = 0; i < MF_DISKS; i++) {
			if (strcmp(node[dev].devname, MDarray[i].label) == 0)
				udasd[i] += util[dev];
		}

		if (strcmp(node[dev].devname, "TR") == 0)
			ulan += util[dev];
	}

	printf("Node      \t%% Utilization\n");
	printf("----      \t--------------\n");
	printf("%s\t%7.4f\n", "Token Ring ", ulan);
	printf("%s\t%7.4f\n", "Desktop PC ", uws / USERS);
	printf("%s\t%7.4f\n", "FileServer ", ufs);

	for (i = 0; i < FS_DISKS; i++) {
		printf("%s%d\t%7.4f\n", "FS Disk", 
			FDarray[i].id, 
			udsk[i]
		);
	}

	printf("%s\t%7.4f\n", "Gateway SNA", ugw);
	printf("%s\t%7.4f\n", "Mainframe  ", umf);

	for (i = 0; i < MF_DISKS; i++) {
		printf("%s%d\t%7.4f\n", "MFrame DASD", 
			MDarray[i].id, 
			udasd[i]
		);
	}
}  /* main */

//-------------------------------------------------------------------------

